(* ::Package:: *)

(*The Package*)

(*Title : SysModel, 
  a Mathematica package for solving system of differential equations with 
application for modeling Biokinetic Systems*)
(* : Author : Guillermo Sanchez*)
(* : Summary : 
      This package provides Mathematica functions for solving linear 
compartmental and physiological model*)
(* : Context : Biokmod`SysModel2`*)
(* : Package Version : 2.1.1*)

(* : Copyright : Copyright 2024. Guillermo Sanchez. 
http : //diarium.usal.es/guillermo/*)

(* : History : Version 1.1 added SystemLTSolve; v .1.2 added sseH2, sseH3, 
  and menors update; v .1.3 added qContinuous and qConstant, 
  moved ssH feature for fitting to a new package,  v 1.4 qMultiple, 
  v1.5 SystemParametricDSolve, Isotopes, 
  v 2.0 Expanded qMultiple and added new function for fitting,
  v 2.1 Modified AcuteInput and SystemDSolved witn DSolve option, 
  v.2.1.1 Added Tiacdes *)

(* : Keywords : Differential equation system, Compartmental Model, 
  Physiological Model, Biokinetic, Pharmacokinetic, Internal Dosimetry, 
  Radiological Protection*)

(*Sources : Jacquez 1985, Andersen 1983, Godfrey 1983*)

(* : Warning :*)

(* : Mathematica Version : Tested with Math 14*)

(* : Limitation : None known.*)

(* : Discussion :*)



BeginPackage["Biokmod`SysModel2`"]

CoefMatrix::usage = "CoefMatrix[n,{coeffs}], gives the matrix of 
coefficients, for n number of retention variables and coeffs are coefficients 
of the matrix. They are given as {{..{i,j,aij},{..},...} where ij is 
position of the coef. in the matrix (i=file, j=column) and are the values 
of this coeff (By default aij = 0).";

CompartMatrix::usage = "CompartMatrix[n,{transcoeff},lambda], gives the
matrix of coefficients,also called  constant transfer coefficients matrix. 
Where n isthe number of compartments and transcoeff are the transfer coefficients, also 
called clearance coeffs. or dissolution rates. They are given as 
{{..{i,j,kij},{..},...} where kij is the transfer coeff., in \!\(t\^-1\),. 
from compartment i to compartment j (By default kij = 0); lambda is the 
radioactive decay constant, in the same unit as the trans. coef. (by default lambda = 0, which 
means thatit is not a radioactive substance.)";

matrixExp::usage = 
"matrixExp[matrixA,t] gives the exponential matrix Exp[A t]";

AcuteInput::usage = "AcuteInput[MatrixCoefs, input, t, x] gives the 
retention in each compartment i at time t (in the same unit that the 
1/transfer of coeffs.) after of a single input {b1,..,bi,..,bn} in the 
compartments {1,..,i,..n} at time t = 0. MatrixCoefs is the coeffs matrix 
{{..,aij,..},{..},...}. It can be given directly or better obtained by using 
the function of this package CompartMatrix or CoefMatrix; x is the symbol 
that represents the retention variables.";

AcuteInput1::usage = "AcuteInput1[MatrixCoefs, input, t, x] Old version of AcuteInput where matrixExp is used instead of 
MatrixExp";


MultiExpInput::usage = "MultiExpInput[MatrixCoefs,incond,input,t, t1, x] 
gives the retention in each compartment i at time t, in the same unit that 
the 1/transfer of coeffs., assuming a multiexponential input {a11 Exp[b11 t]+
...+a1i Exp[b1it],...,an1 Exp[bn1 t]+..+ani Exp[bnit]}; incond {c1,..,ci,..cn} 
are the initial conditions in compartments {1,..,i,..,n}. MatrixCoefs is the 
coeffs matrix {{..,aij,..},{..},...}. It can be given directely or better obtained 
using the function of this package CompartMatrix or CoefMatrix; x is the symbol 
that represents the retention variables.";

SystemDSolve::usage = "SystemDSolve[MatrixCoefs, incond, input, t, t1, x, options] gives 
the solution X(t)= {x1(t), ..., xn(t) at time t1 of a ODE system X'(t) = A X(t) + B(t), 
with initial conditions X(0) = IC , where MatrixCoefs is the 
coeffs matrix A = {{a11,...a1n}, ...,{am1, ..., amn}}; incond are IC = {c1,..,ci,..cn}; 
B(t) ={b1[t],..., bn[t]};  x is the symbol that represents X; with options 
can be choose the method applied for solving the SODE, by default is apply: Method-> 
Automatic, other methods are LaplaceTransform, EigenValues, is applied the 
Eigenvalues-eigensisystem method, and DSolve is applied the Mathematica function DSolve.";

SystemNDSolve::usage = "SystemNDSolve[MatrixCoefs, incond, input, {t, tmin, tmax}, t1, x, options] 
gives the numeric solution X(t)= {x1(t), ..., xn(t) at time t1 of a ODE 
system X'(t) = A X(t) + B(t), with initial conditions X(0) = IC , where; MatrixCoefs is the 
coeffs matrix A = {{a11,...a1n}, ...,{am1, ..., amn}}; ij can be constants of function of t;
incond are IC = {c1,..,ci,..cn}; 
B(t) ={b1[t],..., bn[t]};  x is the symbol that represents X; options are the 
options of NDSolve. 
Note:  This function apply the Mathematica function NDSolve  for solving the 
SODE, it should be used when MatrixCoefs included some coeffs that are not constants 
or when it was not possible obtain the solution of the model using SystemDSolve.";

ParametricSystemNDSolve::usage ="ParametricSystemNDSolve[MatrixCoefs, incond, input, {t, tmin, tmax}, 
 x,  par, opts] gives the numeric solution X(t)= {x1(t), ..., xn(t) at time t1 of a ODE 
system X'(t) = A X(t, par) + B(t), with initial conditions X(0) = IC as function of parameters par, 
where; MatrixCoefs is the  coeffs matrix A = {{a11,...a1n}, ...,{am1, ..., amn}}; 
aij can be constants of function of t; incond are IC = {c1,..,ci,..cn};  
B(t) ={b1[t],..., bn[t]};  x is the symbol that 
represents X; options are the  options of ParametricNDSolve. 
Note:  This function apply the Mathematica function ParametricNDSolve ";

SystemLTSolve::usage = "SystemLTSolve[MatrixCoefs, incond, input, t, s, X, options] gives  
the laplace transform solution X(s)= {X1(s), ..., Xn(s)}  of a ODE system x'(t) = A x(t) + b(t), 
 with initial conditions x(0)  where MatrixCoefs is A = {{a11,...a1n}, ...,{am1, ..., amn}}, 
incond are initial conditions x(0) = {x1(0),....,xn(0)} , and input is b(t) ={b1[t],..., bn[t]}";

Catenary::usage = "Catenary[b, n, t, K, k, lambda] gives the amount  
(activity,mass,concentration, etc.) in the n-th compartment of a one-way 
catenary chain at any time t after a single intake b in the first compartment
of the chain, K is the general symbol applied for indicate the total transfer since one 
compartment and k notate the transfer since any compartments of the catenary chain; 
lambda is the radioactive decay constant, in the same unit that the trans. coef. 
(by default i lambda = 0, which means it that is not a radioactive substance. 
Note: The transfer coefficients are given as ki and Ki where ki is the transfer coeffs, 
in \!\(t\^-1\), from compartment i to compartment i+1 and Ki is the total transfer coeffs. 
from i to others. "; 

ShowODE::usage = "ShowODE[MatrixCoefs,incond,input,t, x] gives system of 
ODE for a compartmental or physiological  model where MatrixCoefs is the 
coeffs matrix {{..,aij,..},{..},...}, it can be given directly  or better obtained by usingthe 
function of this package CompartMatrix or CoefMatrix; incond {c1,..,ci,..cn} are 
the initial conditions in compartments {1,..,i,..,n}; input are the inputs 
{b1[t],..., bn[t]} in compartment {1, .., n}; x is the symbol that 
represents the retention variables.";

qContinuous::usage = "qContinuous[continuous,input-inpulse,t,t1] gives the 
amount (activity, mass, etc) in one compartment or region (i.e Lungs) of 
ICRP66-Extended system for a continuous intake b(t) during a time t1 where is 
known the retention function for a single intake 1 in t=0 (it can be 
calculated using AcuteInput)."; 

qConstant::usage = "qConstant[b,{u[t],t},T, tt] gives the 
retention the time tt since the constant input b started. 
The input happen during a time T; u[t] is the unit-inpulse 
response";

qMultiple::usage = "qMultiple[inputsdata,{u[t],t}, tt] gives the 
retention the time tt for multiples constant:{...,{bi,ti,Ti},...}
and singles{...,{bi,ti},...})inputs,being u[t] the unit-inpulse
response"; 

qMultiple1::usage = "qMultiple1[inputsdata,{u[t],t}, tt] gives the 
retention the time tt for multiples constant:{...,{bi,ti,Ti},...}
and singles{...,{bi,ti},...})inputs,being u[t] the unit-inpulse
response. Its the same that qMultiple, but usually is slower, 
however it recomended be used  when the patter of input function is not ai Exp[- ki t] ";

Modelfit::usage = "Modelfit[rf, list,  sfB] estimates the intake b being rf the response function 
using the bioassay data list:{{t1, m1,u1 },{t2, m2, u2 }, ...}, and the  scattering factor sfB";

Modelfit2::usage = "Modelfit2[{rt1, list1, sfB1}, {rt2, list2, sfB2}] estimate the intake b 
using a biresponse model, being rf1 and r2 the response functions, for two  type of biossay 
and the bioassay data list1: {{t1, m1,u1 },..., {t2, m2, u2 }, ...} and list2 , using the scattering  
factors sfB1 and sfB2 for bioassay type 1 and 2";

ModelfitLSM::usage = "ModelfitLSM[model, list, parameters]estimate the intake {b1,b2, ..] using the LSM
being the model response function, as funcion of b1, b1, ..;  list:{{t1, m1,u1 },{t2, m2, u2 }, ...}, 
and parameters to be fitted: {b1, b2, ..}";

ModelfitLogLSM::usage = "ModelfitLogLSM[model, list, parameters, sfB]bestimate the intake {b1,b2, ..] 
using the LSM (assuming that the uncertainties follow a log.normal distribution)
being the model response function, as funcion of b1, b1, ..;  list:{{t1, m1,u1 },{t2, m2, u2 }, ...}, 
and parameters to be fitted: {{b1, b10}, {b2, b20}, ...} where b10, b20, .... are the starting point to 
find the minimun";

Assayfit::usage = "Assayfit[rf, inputs1, samplevalues, endfirstperiod] being rt the response 
function, inputs: {{s1, T1}, {s2, T2}, ...}, list:{{t1, m1,u1 },{t2, m2, u2 }, ...}, 
A constant intake B1 is assumed during periods: T1..Ti, and B2 during Ti+1 +  Ti+1,  
being endfirstperiod=i";

(*Added in 2.07->*)

qMultipleUnitCte::usage = "qMultipleUnitCte[inputsdata,{uc[t],t}, tt] gives the 
retention or excreption the time tt for multiplesconstant:{...,{bi,ti,Ti},...}
and singles{...,{bi,ti},...})inputs,being uc[t] the unit-inpulse
response for a constant intake 1";

qMultipleUnitCte1::usage = "qMultipleUnitCte1[inputsdata,{uc[t],t}, tt] gives the 
retention or excreption the time tt for multiples constant:{...,{bi,ti,Ti},...}  
and singles{...,{bi,ti},...})inputs (showing the separate contribution T1, T1+T2, .. )
being ud[t] the unit-inpulse response for a constant intake 1";  

qMultipleSingle::usage = "qMultipleSingle[inputsdata,u, k] gives the 
retention or excreption the time tn+k for multiples single intake 
{{b1,},...,{bi,ti},...,{bn,tn}}) being k the number of days after happens the last intake bn,
and u[t] the unit-inpulse response for a single intake 1";

qRandom::usage = "qRandom[b1, T, inp, mu, sig, z, n, m , t_:t] 
gives the IRF for an random imput with mean mu and standard deviation sig."; 

qRandomSingle::usage = "qRandomSingle[rf, workingdays, k, \[Mu]b, sb, zb] being 
rf the response funtion, workingdays:{1,1,1,1,0,0,0}, k: days after the last intake, 
\[Mu]b, sb, zb:2]";

Modelfit3::usage = "Modelfit3[data1, model1, , data2, model2, ...,t] (If the number of bioassay is lower 
than 3 better use Modelfit or Modelfit2 ";

ModelfitX2::usage =  "ModelfitX2[{param}, list1, model1, response funtion, t] "

(*<-Added in 2.07*)
IsotopeDecayModes::usage ="For an isotope iso gives the Decay Modes, Decay Mode Symbols, Branching Ratios, and  DecayEnergies";

IsotopeChainPlot::usage ="For an isotope iso gives the decay chains ";

IsotopeChainData::usage ="For an isotope iso gives a table shown the main chain of Daughter Nuclides, Symbol, Half Life, Branching Ratios";
 
Disintegrations::usage = "Tiacdes[unit of time: {sec, min, hour, day}, impulse response functions {r1(t), ....rk(t)}, t,  moments of time to be computed the 
accumulated disintegrations: {t1, ..., tj} ]  gives the  accumulated disitintegrations in monents {t1, ..., tj}";

Tiacdes::usage = "Tiacdes[unit of time: {sec, min, hour, day}, impulse response functions {r1(t), ....rk(t)}, t,  moments of time to be computed the 
accumulated disintegrations: {t1, ..., tj} ]  gives the TIAC and the accumulated disitintegrations or TIA in monents {t1, ..., tj}"  

Options[SystemDSolve] = Method -> "Automatic";

Options[SystemNDSolve] = Options[NDSolve];



Begin["`Private`"]

CompartMatrix[(n_)?IntegerQ, lst_?MatrixQ, lambda_:0] := 
    Module[{k, A}, Apply[Set, ({k @@ Take[#1, 2], Last[#1]} &) /@ lst, {1}];
      k[i_, j_] := 0; 
      A = DiagonalMatrix[Table[-Sum[k[i, j], {j, 0, n}], {i, 1, n}]] + 
          Table[k[j, i], {i, 1, n}, {j, 1, n}] - lambda*IdentityMatrix[n]];

CoefMatrix[(n_)?IntegerQ, (lst_)?MatrixQ] := 
    Module[{i, j, k}, 
      Normal[SparseArray[lst /. {i_, j_, k_} -> {i, j} -> k, {n, n}]]];

matrixExp[matrixA_?MatrixQ, t_] := 
    Module[{eval, evec, m1}, {eval, evec} = Eigensystem[matrixA];
      m1 = 
        Transpose[evec] . DiagonalMatrix[Exp[eval*t]] . Inverse[
            Transpose[evec]]];

AcuteInput[matrixA_?MatrixQ, input_List, t_, x_] := 
  Module[{r}, r = Dot[MatrixExp[matrixA t], input];
    Thread[Subscript[x, #1][t] & /@ Range[Length[input]] -> r]];
    
AcuteInput1[matrixA_?MatrixQ, input_List, t_, x_] := 
  Module[{r}, r = Dot[matrixExp[matrixA, t], input];
    Thread[Subscript[x, #1][t] & /@ Range[Length[input]] -> r]];
    
(*v. 2.1  MatrixExp is used instead of matrixExp *)
MultiExpInput[matrixA_List, incond_List, input_List, t_, t1_, x_] := 
    Module[{m1, m2, r, tau, a, b, c}, m1 = MatrixExp[matrixA t];
        m2 = 
          Chop[ExpandAll[Evaluate[m1 /. t -> tau] . input /. t -> t - tau]] /. 
                E^((b_.)*t + (c_.)*tau + (d_.)*tau) -> 
                  E^(b*t + (c + d)*tau) /. 
              Exp[(c_.)*tau] -> (E^(c*t) - 1)/c /. 
            E^((b_.)*t + (c_.)*tau) -> (E^(b*t)*(E^(c*t) - 1))/c;
        r = ExpandAll[m1 . incond + m2];
        Thread[Subscript[x, #1][t] & /@ Range[Length[input]] -> r] /. 
          t -> t1] /; 
      Plus @@ (input /. a_.*Exp[b_.*t] /; FreeQ[{a, b}, t] -> 0) === 0;

SystemDSolve["Automatic", matrixA_, incond_List, input_List, t_, t1_, x_, 
        opts___?OptionQ] /; MatrixQ[matrixA, NumericQ] := 
    Module[{integrate, t2, y, z, a, b, c, d, tau, m1, m2, m3, m4, r}, 
      integrate[y_ + z_, t2_] := integrate[y, t2] + integrate[z, t2];
      integrate[c_ y_, t2_] := 
        c*integrate[y, t2] /; FreeQ[c, t2] && FreeQ[c, tau];
      integrate[E^((b_.)*t2_ + (c_.)*tau), 
          t2_] := (E^(b*t2)*(E^(c*t2) - 1))/c /; 
          FreeQ[{b, c}, t2] && FreeQ[{b, c}, tau];
      integrate[Exp[(a_.)*tau], t2_] := (E^(a*t2) - 1)/a /; 
          FreeQ[a, t2] && FreeQ[a, tau];
      integrate[c_, t2_] := c*t2 /; FreeQ[c, t2] && FreeQ[c, tau];
      integrate[a_, t2_] := Integrate[a, {tau, 0, t2}];
      m1 = MatrixExp[matrixA t]; m2 = m1 /. t -> tau;
      m3 = input /. t -> t - tau;
      m4 = 
        Chop[ExpandAll[m2 . m3]] /. 
          E^((b_.)*t2_ + (c_.)*tau + (d_.)*tau) -> E^(b*t2 + (c + d)*tau);
      r = Chop[ExpandAll[m1 . incond + Map[integrate[#1, t] &, m4, {1}]]];
      Thread[Subscript[x, #1][t] & /@ Range[Length[input]] -> r] /. t -> t1];

SystemDSolve["LaplaceTransform", matrixA_?MatrixQ, incond_List, input_List, 
      t_, t1_, x_, opts___?OptionQ] := 
    Module[{B, A1, R, P, r, n, s}, A1 = matrixA; n = Dimensions[A1][[1]];
      B = s*IdentityMatrix[n] - A1; R = Inverse[B]; P = Apart[R];
      r = Table[LaplaceTransform[Extract[input, i], t, s], {i, 1, n}];
      R = 
        Chop[ExpandAll[
            InverseLaplaceTransform[P, s, t] . incond + 
              InverseLaplaceTransform[P . r, s, t]]];
      Thread[Subscript[x, #1][t] & /@ Range[n] -> R] /. t -> t1];

SystemDSolve["EigenValues", matrixA_?MatrixQ, incond_List, input_List, t_, 
      t1_, x_, opts___?OptionQ] := 
    Module[{integrate, t2, y, z, a, b, c, d, tau, m1, m2, m3, m4, r}, 
      integrate[y_ + z_, t2_] := integrate[y, t2] + integrate[z, t2];
      integrate[c_ y_, t2_] := 
        c*integrate[y, t2] /; FreeQ[c, t2] && FreeQ[c, tau];
      integrate[a_, t2_] := Integrate[a, {tau, 0, t2}];
      m1 = MatrixExp[matrixA t]; m2 = m1 /. t -> tau;
      m3 = input /. t -> t - tau;
      m4 = 
        Chop[ExpandAll[m2 . m3]] /. 
          E^((b_.)*t2_ + (c_.)*tau + (d_.)*tau) -> E^(b*t2 + (c + d)*tau);
      r = Chop[ExpandAll[m1 . incond + Map[integrate[#1, t] &, m4, {1}]]];
      Thread[Subscript[x, #1][t] & /@ Range[Length[input]] -> r] /. t -> t1];

(*v. 2.1 *)

SystemDSolve["DSolve", matrixA_, incond_List, input_List, t_, t1_, x_, (opts___)?OptionQ] /; MatrixQ[matrixA, NumericQ] := 
  Module[{X, t2, system, sol}, X[t2_] = (Subscript[x, #1][t2] & ) /@ Range[Length[incond]]; system = Derivative[1][X][t] == matrixA . X[t] + input; 
    sol = ExpandAll[DSolve[{system, X[0] == incond}, X[t], t]]; sol /. t -> t1];

SystemDSolve[matrixA_?MatrixQ, incond_List, input_List, t_, t1_, x_, 
      opts___?OptionQ] := 
    Module[{meth}, meth = Method /. {opts} /. Options[SystemDSolve];
      SystemDSolve[meth, matrixA, incond, input, t, t1, x, opts]];

Catenary[b_, n_, t_, K_, k_, lambda_:0] := 
    Block[{p, 
        j}, (b*Sum[
              1/(E^(Subscript[K, j]*t)*(Product[
                          Subscript[K, p] - Subscript[K, j], {p, 1, j - 1}]*
                        Product[
                          Subscript[K, p] - Subscript[K, j], {p, j + 1, 
                            n}])), {j, 1, n}]*
            Product[Subscript[k, j], {j, 1, n - 1}])/E^(lambda*t)];

ShowODE[matrixA_, incond_, input_, t_, x_] := 
    Module[{n, n1, A, B, c}, n = Dimensions[matrixA][[1]]; n1 = Range[n];
      A = (Derivative[1][Subscript[x, #1]][t] &) /@ n1;
      B = (Subscript[x, #1][t] &) /@ n1 . Transpose[matrixA] + input;
      c = (Subscript[x, #1][0] &) /@ n1; 
      Join[Thread[A == B], Thread[c == incond]]];

SystemNDSolve[matrixA_, incond_, input_, {t_, tmin_, tmax_}, t1_, x_, 
      opts___] := 
    Module[{B, A1, R, P, n, i, opt1}, A1 = matrixA; n = Dimensions[A1][[1]];
      R = 
        Join[Thread[
            Table[Derivative[1][Subscript[x, i]][t], {i, 1, n}] == 
              Table[Subscript[x, i][t], {i, 1, n}] . Transpose[matrixA] + 
                input], 
          Thread[Table[Subscript[x, i][0], {i, 1, n}] == incond]];
      B = Table[Subscript[x, i][t], {i, 1, n}];
      Flatten[NDSolve[R, B, {t, tmin, tmax}, opts]] /. t -> t1];

SystemLTSolve[matrixA_List, incond_List, input_List, t_, s_, X_] := 
    Module[{B, A1, R, R1, P, r, n}, A1 = matrixA; n = Dimensions[A1][[1]];
      B = s*IdentityMatrix[n] - A1; R = Inverse[B]; P = Apart[R];
      r = LaplaceTransform[input, t, s];
      R1 = Dot[P, incond] + Dot[P, r];
      Thread[Subscript[X, #1][s] & /@ Range[n] -> R1]];

ParametricSystemNDSolve[matrixA_, incond_, input_, {t_, tmin_, tmax_}, x_, par_, opts___] := 
   Module[{B, A1, R, P, n, i, opt1}, A1 = matrixA; n = Dimensions[A1][[1]]; 
     R = Join[Thread[Table[Derivative[1][Subscript[x, i]][t], 
         {i, 1, n}] == Dot[Table[Subscript[x, i][t], {i, 1, n}], Transpose[matrixA]] + 
          input], Thread[Table[Subscript[x, i][0], {i, 1, n}] == incond]]; 
     B = Table[Subscript[x, i], {i, 1, n}]; 
     Flatten[ParametricNDSolve[R, B, {t, tmin, tmax}, par, opts]]]; 

qContinuous[input_, acute_List, t_, t1_] := 
    Module[{integrate, t2, y, z, a, b, c, d, tau, m1, m2, m3, m4, r, i}, 
      integrate[y_ + z_, t2_] := integrate[y, t2] + integrate[z, t2];
      integrate[c_*y_, t2_] := 
        c*integrate[y, t2] /; FreeQ[c, t2] && FreeQ[c, tau];
      integrate[E^((b_.)*t2_ + (c_.)*tau), 
          t2_] := (E^(b*t2)*(E^(c*t2) - 1))/c /; 
          FreeQ[{b, c}, t2] && FreeQ[{b, c}, tau]; 
      integrate[Exp[(a_.)*tau], t2_] := (E^(a*t2) - 1)/a /; 
          FreeQ[a, t2] && FreeQ[a, tau]; 
      integrate[c_, t2_] := c*t2 /; FreeQ[c, t2] && FreeQ[c, tau]; 
      integrate[a_, t2_] := Integrate[a, {tau, 0, t2}] /; ! FreeQ[a, tau]; 
      m1 = acute; m2 = m1 /. t -> tau;
      m3 = input /. t -> t - tau; 
      m4 = Chop[ExpandAll[m2*m3]] /. 
          E^((b_.)*t2_ + (c_.)*tau + (d_.)*tau) -> E^(b*t2 + (c + d)*tau);
      r = Chop[ExpandAll[(integrate[#1, t] &) /@ m4]]; r /. t -> t1];

qConstant[input_, {r_, t_}, t1_, T_] := 
    Module[{xc, xc1, xc2}, xc = qContinuous[1, {r}, t, t][[1]]; 
      xc1 = xc /. t -> t1; xc2 = xc /. t -> t1 - T;
      Piecewise[{{"t1 must be non negative", t1 < 0}, {input xc1 /. t -> t1, 
            0 <= t1 <= T}, {input (xc1 - xc2), t1 > T}}]];
(*New functionsb 2.1.1*)             

Disintegrations[unittime_, solsystemEq_, t1_, timepoints_] := Module[{unit, funit, funit1, num, tapend, tau, sol, sol1, sol2, v}, unit = unittime; 
funit = Which[unit == "day", 3600*24, unit == "hour", 3600, unit == "min", 60, unit == "sec", 1]; 
num = Length[solsystemEq]; sol = Integrate[solsystemEq, {t1, 0, tau}]; sol1 = sol /. tau -> timepoints; sol2 = funit*sol1; 
v = Join[{timepoints*unittime "(Bqs)"}, sol2]; 
 MapThread[Prepend, {v, Join[{"Compart"}, Range[num]]}]]

Tiacdes[unittime_, solsystemEq_, t1_, timepoints_] := 
 Module[{unit, funit, funit1, num, tapend, tau, sol, sol1, sol2, v}, 
     unit = unittime; 
  funit = Which[unit == "day", 3600*24, unit == "hour", 3600, 
    unit == "min", 60, unit == "sec", 1]; 
      funit1 = 
   Which[unit == "day", 24, unit == "hour", 1, unit == "min", 1/60, 
    unit == "sec", 1/3600]; 
      num = Length[solsystemEq]; 
  sol = Integrate[solsystemEq, {t1, 0, tau}]; 
  sol1 = sol /. tau -> Prepend[timepoints, 10000]; 
      sol2 = (Prepend[Table[funit, Length[timepoints]], 
        funit1]*#1 & ) /@ sol1; 
      v = Join[{Prepend[timepoints*unittime "(Bqs)", "TIAC(h)"]}, sol2]; 
  MapThread[Prepend, {v, Join[{"Compart"}, Range[num]]}]];

 
 (*Updated b 2.07*)
 

qMultiple[inputdata_,  {ff_, tt_}, ts_] := 
    Module[{u, uc, tau,r, rr, rt, bi, t1, ti, Ti, t2,  t3}, u[t2_] = ff /. tt -> t2; 
   uc[t3_] =  qContinuous[1, {u[tau]}, tau, t3][[1]];
       r[t1_, Ti_] := Piecewise[{{0, t1 < 0}, {uc[t1], 0 <= t1 <= Ti}, 
             {uc[t1] - uc[t1 - Ti], t1 > Ti}}]; 
       rr[bi_, ti_] := Piecewise[{{0, t1 - ti < 0}, {bi*u[t1 - ti], 
           t1 - ti >=  0}}]; 
       rr[bi_, ti_, Ti_] := (bi/Ti)*r[t1 - ti, Ti]; 
       rt = rr @@@ inputdata /. t1 -> ts; Total[rt]];

qMultiple1[inputdata_,  {ff_, tt_}, ts_] := 
  Module[{u, uc,tau,rr,rt, bi, t1, ti, Ti, t2,  t3}, u[t2_] = ff /. tt -> t2; 
uc[t3_] = Integrate[ff,{tt, 0, t3}];
    r[t1_, Ti_] := Piecewise[{{0, t1 < 0}, {uc[t1], 0 <= t1 <= Ti}, 
       {uc[t1] - uc[t1 - Ti], t1 > Ti}}]; 
    rr[bi_, ti_] := Piecewise[{{0, t1 - ti < 0}, {bi*u[t1 - ti], 
    t1 - ti >=  0}}]; 
    rr[bi_, ti_, Ti_] := (bi/Ti)*r[t1 - ti, Ti]; 
    rt=rr@@@inputdata /. t1 -> ts; Total[rt]];

qMultipleUnitCte[inputdata_,  {ff_, tt_}, t1_] := 
    Module[{u, r, rr, bi, t, ti, Ti, t2}, u[t2_] = ff /. tt -> t2; 
       r[t_, Ti_] := Piecewise[{{0, t < 0}, {u[t], 0 <= t <= Ti}, 
             {u[t] - u[t - Ti], t > Ti}}]; 
       rr[bi_, ti_] := Piecewise[{{0, t - ti < 0}, {bi*Derivative[1][u][t - ti], 
           t - ti >=  0}}]; 
       rr[bi_, ti_, Ti_] := (bi/Ti)*r[t - ti, Ti]; 
       Total[rr@@@inputdata /. t -> t1]];
    
    
qMultipleUnitCte1[inputdata_, {ff_, tt_}, t1_] := Module[{u, r, rr, bi, t, ti, Ti, t2, list1}, 
   u[t2_] = ff /. tt -> t2; r[t_, Ti_] = Piecewise[{{0, t < 0}, {u[t], 0 <= t <= Ti}, 
       {u[t] - u[t - Ti], t > Ti}}]; rr[bi_, ti_] := 
     Piecewise[{{0, t - ti < 0}, {bi*Derivative[1][u][t - ti], t - ti >= 0}}]; 
    rr[bi_, ti_, Ti_] := (bi/Ti)*r[t - ti, Ti]; 
    list1 = Apply[rr, inputdata, {1}] /. t -> t1; 
    ListConvolve[ConstantArray[1, Length[inputdata]], list1, {-Length[inputdata], 1}, 0]];
    
    
qMultipleSingle[list_, f_, n_] := 
   Module[{list1, listb, listt}, list1 = ({0, #} & /@ Range[0, n]) /. ({0, #[[2]]} -> # & /@ list); {listb, listt} = Transpose[list1]; 
     Transpose[{listt, ListConvolve[listb, (f[#1] & ) /@ listt, {-n-1, 1}, 0]}]];
  
qRandom[b1_, T_, inp_, mu_, sig_, z_, n_Integer, m_Integer, t_:t] := 
   Module[{r, rc, T1, list1, r1, r1b, r2, t1, u1, uu1, ub, int}, r[t1_] = inp /. t -> t1; 
     rc[T1_] = (b1/T)*Integrate[r[t1], {t1, T1, T + T1}]; list1 = PadRight[{1}, n, 
     PadRight[PadRight[{1}, m, 1], 7]]; 
     r1 = ListConvolve[list1, (r[#1] & ) /@ Range[n], {-n, 1}, 0]; 
     r1b = (rc[#1] & ) /@ Range[n]; u1 = mu*r1 + r1b; 
     uu1 = Drop[u1, 49]; r2 = ListConvolve[list1, (r[#1]^2 & ) /@ Range[n], {-n, 1}, 0]; ub = sig^2*r2; 
     int = Drop[ListCorrelate[{1, 1, 1, 1, 1, 1, 1}/7, z*Sqrt[ub], {-7, 7}], 43]; 
     Transpose[{Drop[Range[n], 49], uu1, int, uu1 + int, uu1 - int}]]; 

qRandom[inp_, mu_, sig_, z_, n_Integer, m_Integer, t_:t] := 
   Module[{r, list1, r1, r2, t1, u1, uu1, ub, int}, r[t1_] = inp /. t -> t1; 
     list1 = PadRight[{1}, n, PadRight[PadRight[{1}, m, 1], 7]]; 
     r1 = ListConvolve[list1, (r[#1] & ) /@ Range[n], {-n, 1}, 0]; 
     u1 = mu*r1; 
     uu1 = Drop[u1, 49]; r2 = ListConvolve[list1, (r[#1]^2 & ) /@ Range[n], {-n, 1}, 0]; ub = sig^2*r2; 
     int = Drop[ListCorrelate[{1, 1, 1, 1, 1, 1, 1}/7, z*Sqrt[ub], {-7, 7}], 43]; 
     Transpose[{Drop[Range[n], 49], uu1, int, uu1 + int, uu1 - int}]]; 

qRandomSingle[rf_, workingdays_, k_, \[Mu]b_, sb_, zb_] := 
  Module[{totaldays, days1, workingdays1, ret, rr, rsq, 
    rsqroot, interval}, totaldays = Length[workingdays]; 
    days1 = totaldays + k; workingdays1 = 
     PadRight[workingdays, days1]; 
    ret = (rf[#1] & ) /@ Range[days1]; 
    rr = ListConvolve[workingdays1, ret, {-days1, 1}, 0]; 
    rsq = ListConvolve[workingdays1, ret^2, {-days1, 1}, 
      0]; rsqroot = Sqrt[rsq]; interval = zb*sb*rsqroot; 
    {"Upper Limit" -> Transpose[{Range[days1], 
        \[Mu]b*rr + interval}], "Average" -> 
      Transpose[{Range[days1], \[Mu]b*rr}], 
     "Lower Limit" -> Transpose[{Range[days1], 
        \[Mu]b*rr - interval}]}];

(*<-New functions b 2.07*)

Modelfit[ff_,list_,sfB_]:=Module[{a,b,c1,d1,mean,md,ji2,ji2mv},a=Total[Apply[ff[#1]*(#2/#3^2)&,list,{1}]];
b=Total[Apply[ff[#1]^2/#3^2&,list,{1}]];
c1=Total[Apply[Log[#2/ff[#1]]/((#3/#2)^2+Log[sfB]^2)&,list,{1}]];
d1=Total[Apply[1/((#3/#2)^2+Log[sfB]^2)&,list,{1}]];
mean=a/b;md=Exp[c1/d1];
ji2=Total[Apply[((mean*ff[#1]-#2)/#3)^2&,list,{1}]];
ji2mv=Total[Apply[(Log[md*ff[#1]]-Log[#2])^2/((#3/#2)^2+Log[sfB]^2)&,list,{1}]];
{"Mean"->mean,"sd"->1/Sqrt[b],"MeanML"->md,"Ji2"->ji2,"Ji2ML"->ji2mv}];

Modelfit2[{ff_, list_, sfB_}, {gg_, list2_, sfB2_}] := 
   Module[{a, b, a1, b1, c1, d1, c2, d2, mean, md, ji2, ji2mv}, 
    a = Total[Apply[ff[#1]*(#2/#3^2) & , list, {1}]]; 
     b = Total[Apply[ff[#1]^2/#3^2 & , list, {1}]]; 
     a1 = Total[Apply[gg[#1]*(#2/#3^2) & , list2, {1}]]; 
     b1 = Total[Apply[gg[#1]^2/#3^2 & , list2, {1}]]; 
    c1=Total[Apply[Log[#2/ff[#1]]/((#3/#2)^2+Log[sfB]^2)&,list,{1}]];
d1=Total[Apply[1/((#3/#2)^2+Log[sfB]^2)&,list,{1}]];
  c2=Total[Apply[Log[#2/gg[#1]]/((#3/#2)^2+Log[sfB2]^2)&,list2,{1}]];
d2=Total[Apply[1/((#3/#2)^2+Log[sfB2]^2)&,list2,{1}]];
mean = (a + a1)/(b + b1); md = Exp[(c1 + c2)/(d1 + d2)]; 
     ji2 = Total[Apply[((mean*ff[#1] - #2)/#3)^2 & , list, {1}]] + 
       Total[Apply[((mean*gg[#1] - #2)/#3)^2 & , list2, {1}]]; 
ji2mv=Total[Apply[(Log[md*ff[#1]]-Log[#2])^2/((#3/#2)^2+Log[sfB]^2)&,list,{1}]]+
Total[Apply[(Log[md*gg[#1]]-Log[#2])^2/((#3/#2)^2+Log[sfB2]^2)&,list2,{1}]];
     {"Mean" -> mean, "sd" -> 1/Sqrt[b + b1], "MeanML" -> md, "Ji2" -> ji2, "Ji2ML"->ji2mv}];    
ModelfitLSM[rf_, list_, parameters_] := 
  FindMinimum[Total[Apply[((rf[#1] - #2)/#3)^2 & , list, 
     {1}]], parameters]

ModelfitLogLSM[rf_, list_, parameters_, sfB_] :=  
 FindMinimum[Total[Apply[((Log[rf[#1]] - Log[#2])^2/((#3/#2)^2 + Log[sfB]^2)) & , list, {1}]], parameters]

Assayfit[rs_, inputs1_, samplevalues_, 
    endperiod_] := Module[
    {yy, ss, samplestime, r, rc1, q, rr, X, \[CapitalSigma], M, MInv}, 
    {samplestime, yy, ss} = Transpose[samplevalues]; 
     rc1[t_, {s_, T_}] := Piecewise[
       {{0, t < s + 1}, {NIntegrate[rs[u], {u, 1, t - s}], s <= t <= s + T}, 
        {NIntegrate[rs[u], {u, t + 1 - s - T, t - s}], t > s + T}}]; 
     q[t_]:= 
      (rc1[t, #1] & ) /@ inputs1; rr = (q[#1] & ) /@ samplestime; 
     X = (Apply[Plus, {Extract[#1, {1 ;; endperiod}], Extract[#1, 
    {endperiod + 1 ;; Length[inputs1]}]}, {1}] & ) /@ rr; 
     \[CapitalSigma] = DiagonalMatrix[ss^2]; 
     M = Transpose[X] . Inverse[\[CapitalSigma]] . X; 
     MInv = Inverse[M]; 
     Transpose[{MInv . Transpose[X] . Inverse[\[CapitalSigma]] . yy, Sqrt[Diagonal[MInv]]}]];   
     
(*New functionsb 2.07 from Modelfit package->*)    
     
linearfit1[t_] = 0; 
linearfit1[list1_, model1_,  r___, t_] := Module[{a, t1, y1, s1}, 
{t1, y1, s1} = Transpose[list1]; 
Plus@@((model1 /. t -> t1)*(y1/s1^2)) + linearfit1[r, t]];

linearfit2[t_] = 0; 
linearfit2[list1_, model1_,  r___, t_] := Module[{a, t1, y1, s1}, 
{t1, y1, s1} = Transpose[list1]; 
Plus@@((model1 /. t -> t1)^2/s1^2) + linearfit2[r, t]];

Modelfit3[list1_,model1_,  r___, t_] := Module[{a, b}, 
a = linearfit1[list1, model1,  r, t]; b = linearfit2[list1, model1, 
r, t]; Thread[{"Mean", "s"} -> {a/b, 1/Sqrt[b]}]];


fx2e[t_] = 0; 
fx2e[list1_, model1_, r___, t_] := Module[{a, t1, y1, s1}, 
    {t1, y1, s1} = Transpose[list1]; 
     Plus @@ (((model1 /. t -> t1) - y1)^2/s1^2) + fx2e[r, t]]; 

ModelfitX2[{(par__)?NumberQ}, list1_, model1_, r___, t_] := 
  fx2e[list1, model1, r, t];

(*<-New functionsb 2.07 from Modelfit package*)

IsotopeDecayModes[iso_String] := Grid[(IsotopeData[iso, #1] & ) /@ {"DecayModes", "DecayModeSymbols", "BranchingRatios", 
     "DecayEnergies"}]

DaughterNuclides[s_List] := DeleteCases[Union[Join @@ (IsotopeData[#1, "DaughterNuclides"] & ) /@ DeleteCases[s, _Missing]], 
    _Missing]; 

ReachableNuclides[s_List] := FixedPoint[Union[Join[#1, DaughterNuclides[#1]]] & , s]; 

DecayNetwork[iso_List] := Join @@ (Thread[#1 -> DaughterNuclides[{#1}]] & ) /@ ReachableNuclides[iso]; 

IsotopeChainPlot[iso_String] := LayeredGraphPlot[Map[IsotopeData[#1, "Symbol"] & , DecayNetwork[{iso}], {2}], 
VertexLabeling -> True, VertexRenderingFunction -> (Text[Framed[Style[#2, 8], Background -> LightYellow], #1] & )]

IsotopeChainData[x_String] := Module[{vals, isolist}, 
   isolist = Quiet[NestWhileList[IsotopeData[#1, "DaughterNuclides"][[1]] & , x, UnsameQ, 2, 100, -2]]; 
    vals = (Table[IsotopeData[#1, prop], {prop, {"Symbol", "HalfLife", "DaughterNuclides", "BranchingRatios"}}] & ) /@ isolist; 
    Text[Grid[Prepend[vals, {"", "Half Life (s)", "Daughter Nuclides", "BranchingRatios"}], Frame -> All, 
      Background -> {None, {{{LightBlue, White}}, {1 -> LightYellow}}}]]]
End[]

Protect[CoefMatrix, CompartMatrix, matrixExp, AcuteInput, AcuteInput1, MultiExpInput, 
    SystemDSolve, Catenary, ShowODE, SystemNDSolve, SystemLTSolve, ParametricSystemNDSolve,  
    qContinuous, qConstant, qMultiple,qMultiple1,qMultipleUnitCte,qMultipleUnitCte1,qRandom,
    qRandomSingle, Modelfit, Modelfit2, Modelfit3, ModelfitLSM, ModelfitLogLSM, ModelfitX2, Assayfit,
    IsotopeDecayModes, IsotopeChainPlot, IsotopeChainData, Tiacdes];

SetAttributes[Subscript, NHoldRest];

Print["SysModel, version 2.1.1   2025-03-26"]

EndPackage[]

